{
  Warp by fri
  Part 1 - Warp Engine (version 1.0 - 27th July 2009)
}

const
  cred   = $ffffaaaa;
  cgreen = $ffaaffaa;
  cblue  = $ffaaaaff;
  
  maxnum      = 16;   // highest possible playerID on server   |   recommended: (slots+2) or even higher
  maxwarp     = 20;   // maximum number of warp points on map   |   used only in arrays to reduce server load
  spawnflames = true; // spawn flames to show warps' placement?   |   true/false
  warplimit   = 0;    // maximum possible number of teleportations per player   |   0 to disable limiter
  
  warpdir = 'scripts/warp'; // main directories
  creatordir = 'scripts/warpcreator'; //don't put slash ( " / " character ) at the end of these dirs


type tplayer = record
    cool, warps: byte; // cool - cooldown after warping   |   warps - times this player used warp
  end;

type twarp = record
    shape: boolean; // rectangle (true) / circle (false)
    x, y, width, height: single; // if shape = circle then width = radius of circle
    target, team: byte;
  end;

type tdata = record
    cooldown,       // cooldown is added to prevent teleporting back too early
    numwarp: byte;  // number of warps on current map
    still: boolean; // does the player have to stand still to teleport?
    config: string; // directory of current map's config
  end;


var
  active, creatormode: boolean;
  prevmap: string;
  player: array [1..maxnum] of tplayer;
  warp: array [1..maxwarp] of twarp;
  data: tdata;



procedure creator(state: boolean);
  begin
    if (creatormode) and (state) then begin
      writeconsole(0, 'Creator mode is still active.', cgreen);
      exit; // can't re-activate creator mode (but still can re-disable it if something goes wrong)
    end;
    creatormode := state;
    if state then begin
      writeconsole(0, 'Creator mode activated.', cgreen);
      setscriptstate(creatordir, true);
      command('/recompile warpcreator');
    end else begin
      writeconsole(0, 'Creator mode disabled.', cred);
      writeconsole(0, 'Use /reload to apply changes.', cred);
      setscriptstate(creatordir, false);
      command('/recompile warpcreator');
    end;
  end;



procedure teleport(id, warpid: byte);
  begin
    if player[id].cool = 0 then begin
      if ((data.still) and (getplayerstat(id, 'velx') = 0) and (getplayerstat(id, 'vely') = 0)) or not (data.still) then begin
        moveplayer(id, warp[warp[warpid].target].x, warp[warp[warpid].target].y);
        player[id].cool := data.cooldown;
        if warplimit > 0 then begin
          player[id].warps := player[id].warps + 1;
          if warplimit = player[id].warps then begin
            writeconsole(id, '  --- warp limit reached ---       ', cred);
            writeconsole(id, 'You can''t teleport on this map anymore.', cred);
          end;
        end;
      end;
    end;
  end;



procedure checkwarps;
  var i, j: byte;
      px, py: single;
  begin
    for i := 1 to maxnum do if getplayerstat(i, 'active') = true then begin
      if (warplimit > 0) and (player[i].warps >= warplimit) then exit;
      getplayerxy(i, px, py);
      for j := 1 to data.numwarp do
        if warp[j].target <> 0 then
          if not warp[j].shape then begin // circle
            if distance(px, py, warp[j].x, warp[j].y) <= warp[j].width then teleport(i, j);
          end else // rectangle
            if (px >= warp[j].x - warp[j].width  / 2) and (px <= warp[j].x + warp[j].width  / 2) and
               (py >= warp[j].y - warp[j].height / 2) and (py <= warp[j].y + warp[j].height / 2) then teleport(i, j);
    end;
  end;



function init: boolean;
  var i: byte;
  begin
    try
      data.config := warpdir + '/data/' + lowercase(CurrentMap) + '.ini';
      if not xfileexists(data.config) then begin result := false; exit; end else begin
        data.numwarp := strtoint(readini(data.config, 'BASE', 'numwarp', '0'));
        if data.numwarp <= 0 then begin
          result := false;
          exit;
        end;
        data.cooldown := strtoint(readini(data.config, 'BASE', 'cooldown', '2')); // loading cooldown
        data.still    := true;                                                    // loading stand-still
          case readini(data.config, 'BASE', 'standstill', '0') of                 // ^
            '0', 'no', 'false', 'disable', 'disabled': data.still := false;       // ^
          end;                                                                    // ^
        for i := 1 to data.numwarp do begin
          warp[i].shape := false;
            case readini(data.config, inttostr(i), 'shape', '0') of
              '1', 'true', 's', 'sq', 'square', 'r', 'rect', 'rectangle', '[]': warp[i].shape := true;
            end;
          warp[i].x      := strtoint(readini(data.config, inttostr(i), 'x', '1'));
          warp[i].y      := strtoint(readini(data.config, inttostr(i), 'y', '1'));
          warp[i].width  := strtoint(readini(data.config, inttostr(i), 'width', '60'));
          if warp[i].shape then warp[i].height := strtoint(readini(data.config, inttostr(i), 'height', '60')); // if shape = rectangle then use height, else use width as radius of circle
          warp[i].target := strtoint(readini(data.config, inttostr(i), 'target', '0'));
            if (warp[i].target < 0) or (warp[i].target > data.numwarp) then warp[i].target := 0;
          warp[i].team   := strtoint(readini(data.config, inttostr(i), 'team', '0'));
            if (warp[i].team < 0) or (warp[i].team > 4) then warp[i].team := 0;
        end;
        for i := 1 to maxnum do begin
          player[i].warps := 0;
          player[i].cool := 0;
        end;
        result := true;
      end;
    except result := false; end;
  end;



procedure onmapchange(newmap: string);
  var i: byte;
  begin
    if (creatormode) and (prevmap <> newmap) then begin
      writeconsole(0, 'Creator mode is still active! Map won''t be changed.', cred);
      writeconsole(0, 'Admin: Use [ /creator 0 ] to exit creator mode.', cred);
      writeln(':: Warp > Creator mode is still active! Map won''t be changed.');
      command('/map ' + prevmap);
      exit;
    end else begin
      prevmap := newmap;
      active := init;
    end;
    if active then writeconsole(0, 'Loaded ' + inttostr(data.numwarp) + iif(data.numwarp = 1, ' warp', ' warps') + '. Have fun!', cgreen);
  end;



procedure activateserver();
  begin
    creatormode := false;
    prevmap := '';
    onmapchange(CurrentMap);
  end;



procedure apponidle(ticks: integer);
  var i: byte;
  begin
    if active then begin
      checkwarps;
      for i := 1 to maxnum do if player[i].cool > 0 then player[i].cool := player[i].cool - 1; // cooldown
      if spawnflames then for i := 1 to data.numwarp do createbullet(warp[i].x, warp[i].y, 0, 0, 0, 5, 30); // flames that help finding warp points
    end;
  end;



function oncommand(id: byte; text: string): boolean;
  begin
    result := false;
    
    if id = 255 then exit; // commands below are reserved for in-game admins only

    if lowercase(text) = '/reload' then begin
      if creatormode then begin
        writeconsole(id, 'Can''t reload with creator mode active. Use /creator 0.', cred);
        exit;
      end;
      active := init;
      if active then writeconsole(0, inttostr(data.numwarp) + iif(data.numwarp = 1, ' warp', ' warps') + ' for ' + prevmap + ' loaded and activated.', cgreen)
        else writeconsole(0, 'Warps NOT loaded, check ' + lowercase(prevmap) + '.ini.', cred);
    end

    else if lowercase(text) = '/pos' then writeconsole(id, 'X: ' + inttostr(round(getplayerstat(id, 'x'))) + '   |   Y: ' + inttostr(round(getplayerstat(id, 'y'))), cblue)

    else if (lowercase(getpiece(text, ' ', 0)) = '/creator') and (length(text) > 9) then
      case lowercase(getpiece(text, ' ', 1)) of
        '1', 'on', 'true', 'enable', 'enter', 'start', 'yes': creator(true);
        '0', 'off', 'false', 'disable', 'exit', 'stop', 'no': creator(false);
        else writeconsole(id, 'Eep, I didn''t understand.', cred);
      end;
  end;



procedure onleavegame(id, team: byte; kicked: boolean);
  begin
    if warplimit > 0 then player[id].warps := 0;
  end;