{
  Warp by fri
  Part 2 - Warp Creator (version 1.0 - 27th July 2009)
   -----
  Important note:
    While Wap Creator is active, you can neither change map nor recompile the script.
    This is to prevent incorrect loading and saving config files.
}

const
  cred   = $ffffaaaa;
  cgreen = $ffaaffaa;
  cblue  = $ffaaaaff;

  maxwarp = 20; // maximum number of warp-points on map

  warpdir = 'scripts/warp'; // main directories
  creatordir = 'scripts/warpcreator'; // don't put slash ( " / " character ) at the end of these dirs

type twarp = record
    shape: boolean; // rectangle (true) / circle (false)
    x, y, width, height: single; // if shape = circle then width = radius of circle
    target, team: byte;
  end;

type tdata = record
    cooldown,       // cooldown is added to prevent teleporting back too early
    numwarp: byte;  // number of warps on current map
    still: boolean; // does the player have to stand still to teleport?
    config: string; // directory of current map's config
  end;


var
  step, edit: shortint;
  warpedit, playercreator: byte;
  editmode: boolean;
  warp: array [1..maxwarp] of twarp;
  data: tdata;


{  step:
  0 - placing warp
  1 - global > cooldown
  2 - global > standstill
  3 - warp > shape (increase data.numwarp before this)
  4 - warp > width
  5 - warp > height (only if shape is not a circle)
  6 - warp > target
  7 - warp > team
  8 - completed
  (writeini goes here)
}


procedure msg(text: string; color: longint);
  begin
    drawtext(playercreator, text, 110, color, 0.10, 40, 380);
  end;



function prevstep(current: shortint): shortint;
  begin
    result := current;
    case current of
      1, 2, 4, 5, 7: result := current - 1;
      3: begin
          result := 0;
          warpedit := 0;
        end;
      6: result := iif(warp[warpedit].shape, 5, 4);
    else exit; end;
  end;



function nextstep(current: shortint): shortint;
  begin
    result := current;
    case current of
      0: result := 3;
      1, 3, 5, 6, 7: result := current + 1;
      2: result := 0;
      4: if warp[warpedit].shape then result := 5
        else begin
          writeconsole(playercreator, 'Warp is a circle. Skipping third step.', cgreen);
          warp[warpedit].height := 60;
          result := 6;
        end;
    end;
  end;



procedure savewarp(warpid: byte);
  var section: string;
  begin
    section := inttostr(warpid);
    writeini(data.config, section, 'team', inttostr(warp[warpid].team));
    writeini(data.config, section, 'target', inttostr(warp[warpid].target));
    writeini(data.config, section, 'height', iif(warp[warpid].shape, floattostr(warp[warpid].height), '60'));
    writeini(data.config, section, 'width', floattostr(warp[warpid].width));
    writeini(data.config, section, 'y', floattostr(warp[warpid].y));
    writeini(data.config, section, 'x', floattostr(warp[warpid].x));
    writeini(data.config, section, 'shape', iif(warp[warpid].shape, 'rectangle', 'circle'));
  end;



procedure savebase;
  begin
    writeini(data.config, 'BASE', 'standstill', iif(data.still, '1', '0'));
    writeini(data.config, 'BASE', 'cooldown', inttostr(data.cooldown));
    writeini(data.config, 'BASE', 'numwarp', inttostr(data.numwarp));
  end;



procedure deletewarp(warpid: byte);
  var i: byte;
  begin
    for i := 1 to data.numwarp do
      if i > warpid then begin
        warp[i-1].shape  := warp[i].shape;
        warp[i-1].x      := warp[i].x;
        warp[i-1].y      := warp[i].y;
        warp[i-1].width  := warp[i].width;
        warp[i-1].height := warp[i].height;
        warp[i-1].target := warp[i].target;
        warp[i-1].team   := warp[i].team;
      end;
    for i := 1 to data.numwarp do
      if warp[i].target = warpid then begin
        warp[i].target := 0;
      end else
      if warp[i].target > warpid then begin
        warp[i].target := warp[i].target - 1;
      end;
    writefile(data.config, '');
    data.numwarp := data.numwarp - 1;
    savebase;
    for i := 1 to data.numwarp do savewarp(i);
  end;


procedure init; // loading config
  var i: byte;
  begin
    data.config := warpdir + '/data/' + lowercase(CurrentMap) + '.ini';
    if not xfileexists(data.config) then begin
      writeini(data.config, 'BASE', 'numwarp', '0');
      data.numwarp := 0;
      step := 1;
      warpedit := 1;
      writeconsole(playercreator, 'Created file [ ' + data.config + ' ].', cgreen);
      writeconsole(playercreator, 'Please enter base config values.', cgreen);
      exit;
    end else begin
      data.numwarp := strtoint(readini(data.config, 'BASE', 'numwarp', '0'));
      if data.numwarp <= 0 then exit;
      step := 0;
      data.cooldown := strtoint(readini(data.config, 'BASE', 'cooldown', '2'));
      data.still    := true;
        case readini(data.config, 'BASE', 'standstill', '0') of
          '0', 'no', 'false', 'disable', 'disabled': data.still := false;
        end;
      for i := 1 to data.numwarp do begin
        warp[i].shape := false;
          case readini(data.config, inttostr(i), 'shape', '0') of
            '1', 'true', 's', 'sq', 'square', 'r', 'rect', 'rectangle', '[]': warp[i].shape := true;
          end;
        warp[i].x      := strtoint(readini(data.config, inttostr(i), 'x', '1'));
        warp[i].y      := strtoint(readini(data.config, inttostr(i), 'y', '1'));
        warp[i].width  := strtoint(readini(data.config, inttostr(i), 'width', '10'));
        if warp[i].shape then warp[i].height := strtoint(readini(data.config, inttostr(i), 'height', '60')); // if shape = rectangle then use height, else use width as radius of circle
        warp[i].target := strtoint(readini(data.config, inttostr(i), 'target', '0'));
          if (warp[i].target < 0) or (warp[i].target > data.numwarp) then warp[i].target := 0;
        warp[i].team   := strtoint(readini(data.config, inttostr(i), 'team', '0'));
          if (warp[i].team < 0) or (warp[i].team > 4) then warp[i].team := 0;
      end;
    end;
  end;



procedure activateserver();
  begin
    playercreator := 0;
    init;
  end;



procedure apponidle(ticks: integer); // 0.866
  var tempw, temph: single;
  begin
    if playercreator = 0 then begin
      msg('Use [ /start ] to become a creator.', cgreen);
      exit;
    end;
    if editmode then // spawning flames around warp that is currently being edited
      if warp[warpedit].shape then begin // rectangle: flames on the corners
        tempw := warp[warpedit].width / 2;
        temph := warp[warpedit].height / 2;
        createbullet(warp[warpedit].x - tempw, warp[warpedit].y - temph, 0, 0, 0, 5, 30);
        createbullet(warp[warpedit].x - tempw, warp[warpedit].y + temph, 0, 0, 0, 5, 30);
        createbullet(warp[warpedit].x + tempw, warp[warpedit].y + temph, 0, 0, 0, 5, 30);
        createbullet(warp[warpedit].x + tempw, warp[warpedit].y - temph, 0, 0, 0, 5, 30);
      end else begin // circle: flames on the edge
        tempw := warp[warpedit].width / 2;
        createbullet(warp[warpedit].x, warp[warpedit].y, 0, 0, 0, 5, 30); // center of warp
        createbullet(warp[warpedit].x, warp[warpedit].y - warp[warpedit].width, 0, 0, 0, 5, 30);
        createbullet(warp[warpedit].x - 1.73 * tempw, warp[warpedit].y + tempw, 0, 0, 0, 5, 30);
        createbullet(warp[warpedit].x + 1.73 * tempw, warp[warpedit].y + tempw, 0, 0, 0, 5, 30);
      end;
    case step of // spam messages while creating new config
      0: msg('Go somewhere and use [ /warp ]' + chr(13) + chr(10) + 'to place warp there.' + chr(13) + chr(10) + 'Use [ /help ] for commands.', cblue);
      1: msg('Initialization:   step 1 of 2' + chr(13) + chr(10) + 'Specify cooldown for warps (seconds)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      2: msg('Initialization:   step 2 of 2' + chr(13) + chr(10) + 'Specify stand-still (true or false)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      3: msg('Warp no. ' + inttostr(warpedit) + ':   step 1 of 5' + chr(13) + chr(10) + 'Choose shape (circle/rectangle)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      4: msg('Warp no. ' + inttostr(warpedit) + ':   step 2 of 5' + chr(13) + chr(10) + 'Enter ' + iif(warp[warpedit].shape, 'width', 'radius') + ' (pixels)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      5: msg('Warp no. ' + inttostr(warpedit) + ':   step 3 of 5' + chr(13) + chr(10) + 'Enter height (pixels)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      6: msg('Warp no. ' + inttostr(warpedit) + ':   step 4 of 5' + chr(13) + chr(10) + 'Enter target (warp number)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      7: msg('Warp no. ' + inttostr(warpedit) + ':   step 5 of 5' + chr(13) + chr(10) + 'Enter team (team number)' + chr(13) + chr(10) + '[ /inf ] for more information', cgreen);
      8: begin
          msg('Yay! Warp #' + inttostr(warpedit) + ' created!', cgreen);
          savewarp(warpedit);
          data.numwarp := warpedit;
          savebase;
          step := 0;
        end;
      -1: msg('[ editing warp no. ' + inttostr(warpedit) + ' ]' + chr(13) + chr(10) + 'Use [ /edit save ] to save changes.', cblue);
    end;
  end;



procedure showcommands(id: byte);
  begin
    writeconsole(id, '', $00ffaaaa); // spacer
    writeconsole(id, '--- warp creator [ commands ] :', cred);
    writeconsole(id, '/inf          : info about current step', cgreen);
    writeconsole(id, '/inf (number) : info about specified warp', cgreen);
    writeconsole(id, '/edit (number): edit specified warp', cgreen);
    writeconsole(id, '/warp         : place a warp at your position', cgreen);
    writeconsole(id, '                (also while editing)', cgreen)
    writeconsole(id, '/back         : go back one step (while adding)', cgreen);
    writeconsole(id, '/creator 0    : disable creator mode', cgreen);
  end;



procedure showeditcommands(id: byte);
  begin
    writeconsole(id, '', $00ffaaaa); // spacer
    writeconsole(id, '--- warp creator [ edit commands ] :', cred);
    writeconsole(id, '/edit shape (circle/rectangle)', cgreen);
    writeconsole(id, '/edit x (pixels)', cgreen);
    writeconsole(id, '/edit y (pixels)', cgreen);
    writeconsole(id, '/edit width  (pixels)', cgreen);
    writeconsole(id, '/edit height (pixels)', cgreen);
    writeconsole(id, '/edit target (warp number)', cgreen);
    writeconsole(id, '/edit team (team number)', cgreen);
    writeconsole(id, '/edit reset', cred);
    writeconsole(id, '/edit save', cred);
    writeconsole(id, '/edit delete', cred);
    writeconsole(id, '', $00aaaaff); // spacer
    writeconsole(id, '/edit still (true/false)', cblue);
    writeconsole(id, '/edit cool (seconds)', cblue);
    writeconsole(id, '', $00aaaaff);
    writeconsole(id, '--- map info :', cred);
    writeconsole(id, 'Number of warps: ' + inttostr(data.numwarp), cgreen);
    writeconsole(id, 'Cooldown time:   ' + inttostr(data.cooldown) + ' seconds', cgreen);
    writeconsole(id, 'Standing still:  ' + iif(data.still, 'required', 'not required'), cgreen);
    if warpedit > 0 then writeconsole(id, 'Currently editing warp number ' + inttostr(warpedit), cblue);
  end;



function oncommand(id: byte; text: string): boolean;
  var warpinfo: byte;
  begin
    result := false;
    
    if lowercase(getpiece(text, ' ', 0)) = '/recompile' then begin
      result := true; // recompiling while editing would be baaad
      writeconsole(id, 'Recompiling blocked - creator mode is active.', cred);
      writeln('Recompiling blocked - creator mode is active.');
      exit;
    end;
    
    if (playercreator > 0) and (id <> playercreator) then exit; // only creator can modify config
    
    text := lowercase(text);
    
    if (text = '/start') and (id <> 255) then begin
      writeconsole(0, idtoname(id) + ' is now a warp creator.', cgreen);
      playercreator := id;
      exit;
    end;
    
    if playercreator = 0 then exit; // it has to be here, not mixed with the one above
    
    if text = '/back' then step := prevstep(step) // going back one step
    
    else if (text = '/warp') and (step = 0) then begin
      warpedit := data.numwarp + 1;
      warp[warpedit].x := round(getplayerstat(id, 'x'));
      warp[warpedit].y := round(getplayerstat(id, 'y'));
      writeconsole(id, 'Warp #' + inttostr(warpedit) + ' placed at (' + floattostr(warp[warpedit].x) + ', ' + floattostr(warp[warpedit].y) + ').', cgreen);
      step := 3;
      exit;
    end
    
    else if (text = '/warp') and (step = -1) then begin
      warp[warpedit].x := round(getplayerstat(id, 'x'));
      warp[warpedit].y := round(getplayerstat(id, 'y'));
      writeconsole(id, 'Warp #' + inttostr(warpedit) + ' placed at (' + floattostr(warp[warpedit].x) + ', ' + floattostr(warp[warpedit].y) + ').', cgreen);
    end
    
    // editing existing warps. serious business.
    else if (text = '/edit') or (text = '/edit help') then begin
      showeditcommands(id);
      exit;
    end
    
    else if copy(text, 0, 11) = '/edit still' then begin
      if length(text) <= 12 then begin
        writeconsole(id, 'Enter value for stand-still (true of false).', cred);
        exit;
      end;
      case getpiece(text, ' ', 2) of
        '1', 'true', 'yes', 'require', 'required': data.still := true;
        '0', 'false', 'no', 'notrequire', 'notrequired', 'disable', 'disabled': data.still := false;
        else begin
          writeconsole(id, 'Eep, I didn''t understand.', cred);
          exit;
        end;
      end;
      writeconsole(id, 'Stand-still set to ' + iif(data.still, 'true.', 'false.'), cgreen);
      savebase;
      exit;
    end
    
    else if copy(text, 0, 10) = '/edit cool' then begin
      if length(text) <= 11 then begin
        writeconsole(id, 'Enter value for cooldown (number of seconds).', cred);
        exit;
      end;
      try
        data.cooldown := strtoint(getpiece(text, ' ', 2));
        if data.cooldown < 0 then data.cooldown := 0;
        writeconsole(id, 'Cooldown set to ' + inttostr(data.cooldown), cgreen);
        savebase;
      except
        writeconsole(id, 'Entry must be a number.', cred);
      end;
      exit;
    end
    
    else if (getpiece(text, ' ', 0) = '/edit') and (length(text) > 6) then begin
      if not editmode then begin
        if step <> 0 then begin
          writeconsole(id, 'Can''t edit new warp if previous one', cred);
          writeconsole(id, 'is not fully configured.', cred);
          exit;
        end;
        try
          warpedit := strtoint(getpiece(text, ' ', 1));
        except
          writeconsole(id, 'Incorrect warp number.', cred);
          exit;
        end;
        if data.numwarp = 0 then begin
          writeconsole(id, 'This map has no warps to edit.', cred);
          exit;
        end;
        if (warpedit <= 0) or (warpedit > data.numwarp) then begin
          writeconsole(id, 'Warp number is out of working range (min: 1, max: ' + inttostr(data.numwarp) + ').', cred);
          exit;
        end;
        editmode := true;
        step := -1;
        writeconsole(id, 'Currently editing warp #' + inttostr(warpedit) + '.', cgreen);
        writeconsole(id, 'Its position is marked with four flames.', cgreen);
        writeconsole(id, 'Warp #' + inttostr(warpedit) + ' XY: [ ' + floattostr(round(warp[warpedit].x)) + '  |  ' + floattostr(round(warp[warpedit].y)) + ' ]', cgreen);
        writeconsole(id, 'Your XY:    ' + iif(warpedit > 9, ' ', '') + '[ ' + floattostr(round(getplayerstat(id, 'x'))) + '  |  ' + floattostr(round(getplayerstat(id, 'y'))) + ' ]', cgreen);
        writeconsole(id, 'Use [ /edit help ] for list of commands.', cred);

      // tsukurimashou, tsukurimashou
      end else
        case getpiece(text, ' ', 1) of
          'clear', 'reset': begin
              warp[warpedit].shape  := true;
              warp[warpedit].x      := 1;
              warp[warpedit].y      := 1;
              warp[warpedit].width  := 10;
              warp[warpedit].height := 10;
              warp[warpedit].target := 0;
              warp[warpedit].team   := 0;
              writeconsole(id, 'Warp #' + inttostr(warpedit) + ': all settings reset.', cgreen);
            end;
          'shape': begin
              if length(text) <= 12 then begin
                writeconsole(id, 'Choose shape of warp #' + inttostr(warpedit) + ':', cred);
                writeconsole(id, '[ /edit shape circle ] or', cgreen);
                writeconsole(id, '[ /edit shape rectangle ]', cgreen);
                exit;
              end;
              case getpiece(text, ' ', 2) of
                '0', 'o', 'c', 'cir', 'circle', 'false', 'round', 'rounded': begin
                    warp[warpedit].shape := false;
                    writeconsole(id, 'Warp #' + inttostr(warpedit) + ': shape set to circle.', cgreen);
                  end;
                '1', 'r', 's', 'sq', 'rect', 'rectangle', '[]': begin
                    warp[warpedit].shape := true;
                    writeconsole(id, 'Warp #' + inttostr(warpedit) + ': shape set to rectangle.', cgreen);
                  end;
                else begin
                  writeconsole(id, 'Eep, I didn''t understand.', cred);
                  exit;
                end;
              end;
            end;
          'x', 'y': begin
              if length(text) <= 8 then begin
                writeconsole(id, 'Enter ' + uppercase(getpiece(text, ' ', 1)) + ' position of warp #' + inttostr(warpedit) + '.', cred);
                writeconsole(id, 'Warp #' + inttostr(warpedit) + ' XY: [ ' + floattostr(round(warp[warpedit].x)) + '  |  ' + floattostr(round(warp[warpedit].y)) + ' ]', cgreen);
                writeconsole(id, 'Your XY: [ ' + floattostr(round(getplayerstat(id, 'x'))) + '  |  ' + floattostr(round(getplayerstat(id, 'y'))) + ' ]', cgreen);
                exit;
              end;
              try
                if getpiece(text, ' ', 1) = 'x' then begin
                  warp[warpedit].x := strtoint(getpiece(text, ' ', 2));
                  writeconsole(id, 'Warp #' + inttostr(warpedit) + ': X position set to ' + floattostr(warp[warpedit].x) + '.', cgreen);
                end else begin
                  warp[warpedit].y := strtoint(getpiece(text, ' ', 2));
                  writeconsole(id, 'Warp #' + inttostr(warpedit) + ': Y position set to ' + floattostr(warp[warpedit].y) + '.', cgreen);
                end;
              except
                writeconsole(id, 'Entry must be a number.', cred);
                exit;
              end;
            end;
          'width', 'radius': begin
              if length(text) <= 12 then begin
                writeconsole(id, 'Enter ' + iif(warp[warpedit].shape, 'width', 'radius') + ' of warp #' + inttostr(warpedit) + '.', cred);
                writeconsole(id, 'Your soldier is about 15 pixels tall.', cgreen);
                exit;
              end;
              try
                warp[warpedit].width := strtoint(getpiece(text, ' ', 2));
                if warp[warpedit].width < 0 then warp[warpedit].width := -warp[warpedit].width;
                writeconsole(id, 'Warp #' + inttostr(warpedit) + ': ' + iif(warp[warpedit].shape, 'width', 'radius') + ' set to ' + floattostr(warp[warpedit].width) + '.', cgreen);
              except
                writeconsole(id, 'Entry must be a number.', cred);
                exit;
              end;
            end;
          'height': begin
              if length(text) <= 13 then begin
                writeconsole(id, 'Enter height of warp #' + inttostr(warpedit) + '.', cred);
                writeconsole(id, 'Your soldier is about 15 pixels tall.', cgreen);
                exit;
              end;
              try
                warp[warpedit].height := strtoint(getpiece(text, ' ', 2));
                if warp[warpedit].height < 0 then warp[warpedit].height := -warp[warpedit].height;
                writeconsole(id, 'Warp #' + inttostr(warpedit) + ': height set to ' + floattostr(warp[warpedit].height) + '.', cgreen);
              except
                writeconsole(id, 'Entry must be a number.', cred);
                exit;
              end;
            end;
          'target': begin
              if length(text) <= 13 then begin
                writeconsole(id, 'Enter target warp of warp #' + inttostr(warpedit) + '.', cred);
                exit;
              end;
              try
                warp[warpedit].target := strtoint(getpiece(text, ' ', 2));
                if warp[warpedit].target < 0 then warp[warpedit].target := -warp[warpedit].target;
                if warp[warpedit].target = 0 then begin
                  writeconsole(id, 'Warp #' + inttostr(warpedit) + ': target = 0, warp disabled.', cgreen);
                  exit;
                end;
                writeconsole(id, 'Warp #' + inttostr(warpedit) + ': target set to ' + inttostr(warp[warpedit].target) + '.', cgreen);
              except
                writeconsole(id, 'Entry must be a number.', cred);
                exit;
              end;
            end;
          'team': begin
              if length(text) <= 11 then begin
                writeconsole(id, 'Enter team which can use warp #' + inttostr(warpedit) + '.', cred);
                writeconsole(id, '[ team 1/2/3/4   |   0 - all teams ]', cgreen);
                exit;
              end;
              try
                warp[warpedit].team := strtoint(getpiece(text, ' ', 2));
                if (warp[warpedit].team < 0) or (warp[warpedit].team > 4) then begin
                  warp[warpedit].team := 0;
                  writeconsole(id, 'No such team. Enter number from range 0..4.', cred);
                  writeconsole(id, 'Team value set to 0.', cred);
                  exit;
                end;
                writeconsole(id, 'Warp #' + inttostr(warpedit) + ': team set to ' + inttostr(warp[warpedit].team) + '.', cgreen);
              except
                writeconsole(id, 'Entry must be a number.', cred);
                exit;
              end;
            end;
          'exit', 'stop', 'quit', 'save', 'ok', 'enter': begin
              savewarp(warpedit);
              editmode := false;
              writeconsole(id, 'Success!', cgreen);
              writeconsole(id, 'Warp #' + inttostr(warpedit) + ' saved, edit mode stopped.', cgreen);
              step := 0;
              exit;
            end;
          'delete': begin
              deletewarp(warpedit);
              writeconsole(id, 'Warp #' + inttostr(warpedit) + ' deleted.', cgreen);
              editmode := false;
              step := 0;
              warpedit := 0;
              exit;
            end;
        end;
    end
    // end of serious business
    
    // list of commands
    else if (text = '/com') or (text = '/command') or (text = '/commands') or (text = '/help') then begin
      showcommands(id);
    end
    
    // step info
    else if (text = '/inf') or (text = '/nfo') then begin
      case step of
        1: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ cooldown ] :', cred);
            writeconsole(id, 'Cooldown is added to every player that warps.', cgreen);
            writeconsole(id, 'It prevents moving players back too early.', cgreen);
            writeconsole(id, 'Suggested value: 2', cgreen);
          end;
        2: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ stand-still ] :', cred);
            writeconsole(id, 'If stand-still is enabled, player mustn''t move', cgreen);
            writeconsole(id, 'in order to use warp. Suggested value: true', cgreen);
          end;
        3: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ shape ] :', cred);
            writeconsole(id, 'Warp can be either circle or rectangle.', cgreen);
            writeconsole(id, 'In next step you''ll enter its radius or width.', cgreen);
            writeconsole(id, 'You can use short forms: c / r', cgreen);
          end;
        4: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ width / radius ] :', cred);
            writeconsole(id, 'Width of rectangle or radius of circle.', cgreen);
            writeconsole(id, 'Your soldier is about 15 pixels tall.', cgreen);
            writeconsole(id, 'Suggested: 60-100 (width) / 30-50 (radius)', cgreen);
          end;
        5: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ height ] :', cred);
            writeconsole(id, 'Height of rectangle inside which warp is placed.', cgreen);
            writeconsole(id, 'Your soldier is about 15 pixels tall.', cgreen);
            writeconsole(id, 'Suggested value: 60 - 100', cgreen);
          end;
        6: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ target ] :', cred);
            writeconsole(id, 'Specify target warp to which this warp', cgreen);
            writeconsole(id, 'will teleport players. Set 0 to disable warp.', cgreen);
          end;
        7: begin
            writeconsole(id, '', $00ffaaaa);
            writeconsole(id, '--- info for [ team ] :', cred);
            writeconsole(id, 'Team which can use this warp.', cgreen);
            writeconsole(id, '0 - everyone; 1 - alpha; 2 - bravo;', cgreen);
            writeconsole(id, '3 - charlie; 4 - delta team', cgreen);
          end;
        0: begin
            showcommands(id);
          end;
        else writeconsole(id, '--- No info for current option.', cred);
      end;
    end
    
    // warp info
    else if ((getpiece(text, ' ', 0) = '/inf') or (getpiece(text, ' ', 0) = '/nfo')) and (length(text) > 5) then begin
      try
        warpinfo := strtoint(getpiece(text, ' ', 1));
      except
        writeconsole(id, 'Incorrect warp number.', cred);
        exit;
      end;
      if (warpinfo <= 0) or (warpinfo > data.numwarp) then begin
        writeconsole(id, 'There''s no such warp.', cred);
        exit;
      end;
      if (step <> 0) and (step <> -1) and (warpinfo = warpedit) then begin
        writeconsole(id, 'Can''t get info of unfinished warp.', cred);
        writeconsole(id, 'Please complete the setup first.', cred);
        exit;
      end;
      writeconsole(id, '', $00ffaaaa);
      writeconsole(id, '--- info for [ warp #' + inttostr(warpinfo) + ' ] :', cred);
      writeconsole(id, 'Shape  : ' + iif(warp[warpinfo].shape, 'rectangle', 'circle'), cgreen);
      writeconsole(id, 'X      : ' + floattostr(warp[warpinfo].x), cgreen);
      writeconsole(id, 'Y      : ' + floattostr(warp[warpinfo].y), cgreen);
      writeconsole(id, iif(warp[warpinfo].shape, 'Width  : ', 'Radius : ') + floattostr(warp[warpinfo].width) + ' pixels', cgreen);
      if warp[warpinfo].shape then writeconsole(id, 'Height : ' + floattostr(warp[warpinfo].height) + ' pixels', cgreen);
      writeconsole(id, 'Target : ' + iif(warp[warpinfo].target = 0, 'warp disabled (target = 0)', 'warp #' + inttostr(warp[warpinfo].target)), cgreen);
      writeconsole(id, 'Team   : ' + inttostr(warp[warpinfo].team), cgreen);
      writeconsole(id, 'Distance from you: ' + floattostr(round(distance(warp[warpinfo].x, warp[warpinfo].y, getplayerstat(id, 'x'), getplayerstat(id, 'y')))) + ' pixels', cred);
    end;
  end;



procedure onplayerspeak(id: byte; text: string);
  begin
    if (playercreator = 0) or ((playercreator > 0) and (id <> playercreator)) then exit; // only creator can modify config

    case step of
      -1, 0, 8: exit;
      1: try
          data.cooldown := strtoint(text);
          if data.cooldown < 0 then data.cooldown := 0;
          writeconsole(id, 'Cooldown set to ' + inttostr(data.cooldown) + ' seconds.', cgreen);
          step := nextstep(step);
        except
          writeconsole(id, 'Entry must be a number.', cred);
          exit;
        end;
      2: begin
          text := lowercase(text);
          case text of
            '1', 'true', 'yes', 'require', 'required', 'enable', 'enabled': data.still := true;
            '0', 'false', 'no', 'notrequire', 'notrequired', 'disable', 'disabled': data.still := false;
            else begin
              writeconsole(id, 'Eep, I didn''t understand.', cred);
              exit;
            end;
          end;
          writeconsole(id, 'Stand-still set to ' + iif(data.still, 'true.', 'false.'), cgreen);
          step := nextstep(step);
        end;
      3: begin
          text := lowercase(text);
          case text of
            '1', 'true', 's', 'sq', 'square', 'r', 'rect', 'rectangle', '[]': warp[warpedit].shape := true;
            '0','false', 'c', 'cir', 'circle', 'round', 'rounded', 'o': warp[warpedit].shape := false;
          else begin
            writeconsole(id, 'Eep, I didn''t understand.', cred);
            exit;
          end;
          end;
          writeconsole(id, 'Shape set to ' + iif(warp[warpedit].shape, 'rectangle.', 'circle.'), cgreen);
          step := nextstep(step);
        end;
      4: try
          warp[warpedit].width := strtoint(text);
          if warp[warpedit].width < 2 then warp[warpedit].width := 2;
          writeconsole(id, iif(warp[warpedit].shape, 'Width', 'Radius') + ' set to ' + floattostr(warp[warpedit].width) + ' pixels.', cgreen);
          step := nextstep(step);
        except
          writeconsole(id, 'Entry must be a number.', cred);
          exit;
        end;
      5: try
          warp[warpedit].height := strtoint(text);
          if warp[warpedit].height < 2 then warp[warpedit].height := 2;
          writeconsole(id, 'Height set to ' + floattostr(warp[warpedit].height) + ' pixels.', cgreen);
          step := nextstep(step);
        except
          writeconsole(id, 'Entry must be a number.', cred);
          exit;
        end;
      6: try
          warp[warpedit].target := strtoint(text);
          if (warp[warpedit].target < 0) or (warp[warpedit].target > data.numwarp) then begin
            warp[warpedit].target := warpedit;
            writeconsole(id, 'Wrong warp number. This warp will teleport to itself.', cred);
            writeconsole(id, 'Use [ /edit ' + inttostr(warpedit) + ' ] at a later time.', cred);
          end else writeconsole(id, iif(warp[warpedit].target = 0, 'Warp disabled (target = 0).', 'Target set to warp #' + floattostr(warp[warpedit].target) + '.'), cgreen);
          step := nextstep(step);
        except
          writeconsole(id, 'Entry must be a number.', cred);
          exit;
        end;
      7: try
          warp[warpedit].team := strtoint(text);
          if (warp[warpedit].team < 0) or (warp[warpedit].team > 4) then begin
            writeconsole(id, 'Incorrect team number. Enter 0/1/2/3/4.', cred);
            exit;
          end;
          writeconsole(id, 'Team set to ' + inttostr(warp[warpedit].team) + '.', cgreen);
          step := nextstep(step);
        except
          writeconsole(id, 'Entry must be a number.', cred);
          exit;
        end;
    end;
  end;



procedure onleavegame(id, team: byte; kicked: boolean);
  begin
    if id = playercreator then playercreator := 0;
  end;